#!/usr/bin/env python

from socket import gethostbyname
import sys

import yaml

from charmhelpers.contrib.openstack.utils import configure_installation_source

from charmhelpers.core.hookenv import (
    Hooks,
    UnregisteredHookError,
    config,
    local_unit,
    log,
    relation_get,
    relation_ids,
    relation_set,
    unit_get
)

from charmhelpers.core.host import (
    restart_on_change,
    service_restart
)

from charmhelpers.fetch import (
    apt_install,
    apt_upgrade,
    configure_sources
)

from contrail_analytics_utils import (
    cassandra_units,
    fix_hostname,
    fix_nodemgr,
    fix_permissions,
    fix_services,
    kafka_units,
    units,
    write_alarm_config,
    write_analytics_api_config,
    write_collector_config,
    write_nodemgr_config,
    write_query_engine_config,
    write_snmp_collector_config,
    write_topology_config,
    write_vnc_api_config
)

PACKAGES = [ "contrail-analytics", "contrail-utils", "contrail-nodemgr",
             "python-jinja2" ]

hooks = Hooks()
config = config()

@hooks.hook("cassandra-relation-changed")
def cassandra_changed():
    # 'port' is used in legacy precise charm
    if not relation_get("rpc_port") and not relation_get("port"):
        log("Relation not ready")
        return
    if not config.get("cassandra-ready"):
        units = len(cassandra_units())
        required = config["cassandra-units"]
        if units < required:
            log("{} cassandra unit(s) ready, require {} more".format(units, required - units))
            return
    config["cassandra-ready"] = True
    cassandra_relation()

@hooks.hook("cassandra-relation-departed")
@hooks.hook("cassandra-relation-broken")
def cassandra_departed():
    if not units("cassandra"):
        config["cassandra-ready"] = False
    cassandra_relation()

@restart_on_change({"/etc/contrail/contrail-collector.conf": ["contrail-collector"],
                    "/etc/contrail/contrail-query-engine.conf": ["contrail-query-engine"],
                    "/etc/contrail/contrail-analytics-api.conf": ["contrail-analytics-api"]})
def cassandra_relation():
    write_collector_config()
    write_query_engine_config()
    write_analytics_api_config()

@hooks.hook("config-changed")
def config_changed():
    vip = config.get("vip")
    for rid in relation_ids("contrail-analytics-api"):
        relation_set(relation_id=rid, vip=vip)

@hooks.hook("contrail-analytics-api-relation-joined")
def contrail_analytics_api_joined():
    relation_set(port=8081, vip=config.get("vip"))

@hooks.hook("contrail-api-relation-changed")
def contrail_api_changed():
    if not relation_get("port"):
        log("Relation not ready")
        return
    contrail_api_relation()

@hooks.hook("contrail-api-relation-departed")
@hooks.hook("contrail-api-relation-broken")
@restart_on_change({"/etc/contrail/contrail-snmp-collector.conf": ["contrail-snmp-collector"],
                    "/etc/contrail/vnc_api_lib.ini": ["contrail-topology"]})
def contrail_api_relation():
    write_snmp_collector_config()
    write_vnc_api_config()

@hooks.hook("contrail-discovery-relation-changed")
def contrail_discovery_changed():
    if not relation_get("port"):
        log("Relation not ready")
        return
    contrail_discovery_relation()

@hooks.hook("contrail-discovery-relation-departed")
@hooks.hook("contrail-discovery-relation-broken")
@restart_on_change({"/etc/contrail/contrail-collector.conf": ["contrail-collector"],
                    "/etc/contrail/contrail-query-engine.conf": ["contrail-query-engine"],
                    "/etc/contrail/contrail-analytics-api.conf": ["contrail-analytics-api"],
                    "/etc/contrail/contrail-alarm-gen.conf": ["contrail-alarm-gen"],
                    "/etc/contrail/contrail-topology.conf": ["contrail-topology"],
                    "/etc/contrail/contrail-snmp-collector.conf": ["contrail-snmp-collector"],
                    "/etc/contrail/contrail-analytics-nodemgr.conf": ["contrail-analytics-nodemgr"]})
def contrail_discovery_relation():
    write_collector_config()
    write_query_engine_config()
    write_analytics_api_config()
    write_alarm_config()
    write_topology_config()
    write_snmp_collector_config()
    write_nodemgr_config()

@hooks.hook("http-services-relation-joined")
def http_services_joined():
    name = local_unit().replace("/", "-")
    addr = gethostbyname(unit_get("private-address"))
    services = [ { "service_name": "contrail-analytics-api",
                   "service_host": "0.0.0.0",
                   "service_port": 8081,
                   "service_options": [ "mode http", "balance leastconn", "option httpchk GET /analytics HTTP/1.0" ],
                   "servers": [ [ name, addr, 8081, "check" ] ] } ]
    relation_set(services=yaml.dump(services))

@hooks.hook("identity-admin-relation-changed")
def identity_admin_changed():
    if not relation_get("service_hostname"):
        log("Relation not ready")
        return
    identity_admin_relation()

@hooks.hook("identity-admin-relation-departed")
@hooks.hook("identity-admin-relation-broken")
@restart_on_change({"/etc/contrail/contrail-snmp-collector.conf": ["contrail-snmp-collector"],
                    "/etc/contrail/vnc_api_lib.ini": ["contrail-topology"]})
def identity_admin_relation():
    write_snmp_collector_config()
    write_vnc_api_config()

@hooks.hook()
def install():
    fix_hostname()
    configure_installation_source(config["openstack-origin"])
    configure_sources(True, "install-sources", "install-keys")
    apt_upgrade(fatal=True, dist=True)
    apt_install(PACKAGES, fatal=True)
    fix_permissions()
    fix_services()
    fix_nodemgr()

@hooks.hook("kafka-relation-changed")
def kafka_changed():
    if not relation_get("port"):
        log("Relation not ready")
        return
    if not config.get("kafka-ready"):
        units = len(kafka_units())
        required = config["kafka-units"]
        if units < required:
            log("{} kafka unit(s) ready, require {} more".format(units, required - units))
            return
    config["kafka-ready"] = True
    kafka_relation()

@hooks.hook("kafka-relation-departed")
@hooks.hook("kafka-relation-broken")
def kafka_departed():
    if not units("kafka"):
        config["kafka-ready"] = False
    kafka_relation()

@restart_on_change({"/etc/contrail/contrail-collector.conf": ["contrail-collector"],
                    "/etc/contrail/contrail-alarm-gen.conf": ["contrail-alarm-gen"]})
def kafka_relation():
    write_collector_config()
    write_alarm_config()

def main():
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log("Unknown hook {} - skipping.".format(e))

@hooks.hook("upgrade-charm")
def upgrade_charm():
    write_collector_config()
    write_query_engine_config()
    write_analytics_api_config()
    write_alarm_config()
    write_topology_config()
    write_snmp_collector_config()
    write_vnc_api_config()
    write_nodemgr_config()
    service_restart("supervisor-analytics")

@hooks.hook("zookeeper-relation-changed")
def zookeeper_changed():
    if not relation_get("port"):
        log("Relation not ready")
        return
    zookeeper_relation()

@hooks.hook("zookeeper-relation-departed")
@hooks.hook("zookeeper-relation-broken")
@restart_on_change({"/etc/contrail/contrail-alarm-gen.conf": ["contrail-alarm-gen"],
                    "/etc/contrail/contrail-topology.conf": ["contrail-topology"],
                    "/etc/contrail/contrail-snmp-collector.conf": ["contrail-snmp-collector"]})
def zookeeper_relation():
    write_alarm_config()
    write_topology_config()
    write_snmp_collector_config()

if __name__ == "__main__":
    main()
